function undisplay(element) {
  element.style.display="none";
}

function add_message(severity, message) {
  message = `<div class="message ${severity}" onclick="undisplay(this)">
<span class="hide-button" onclick="undisplay(this.parentElement)">&times;</span>
${message}
</div>`;
  const message_area = document.getElementById('message-area');
  message_area.insertAdjacentHTML('beforeend', message);
}

/*********************************************************
 * Handling server-sent event stream                     *
 *********************************************************/

const events = new EventSource("events");

events.onerror = function(event) {
  if (event.target.readyState === EventSource.CLOSED) {
    setTimeout(() => {
      add_message("danger", "Connection to server lost.");
    }, 1000);
  }
};

events.addEventListener("game-info", event => {
  const info = JSON.parse(event.data);

  window.game.set_game_info(info);
});

events.addEventListener("player-info", event => {
  const info = JSON.parse(event.data);

  window.game.set_player_info(info);
});

events.addEventListener("player-enter", event => {
  const info = JSON.parse(event.data);

  window.game.set_other_player_info(info);
});

events.addEventListener("player-update", event => {
  const info = JSON.parse(event.data);

  if (info.id === window.game.state.player_info.id)
    window.game.set_player_info(info);
  else
    window.game.set_other_player_info(info);
});

events.addEventListener("game-state", event => {
  const state = JSON.parse(event.data);

  for (let prompt of state.prompts) {
    window.game.add_or_update_prompt(prompt);
  }
});

events.addEventListener("prompt", event => {
  const prompt = JSON.parse(event.data);

  window.game.add_or_update_prompt(prompt);
});

/*********************************************************
 * Game and supporting classes                           *
 *********************************************************/

function copy_to_clipboard(id)
{
  const tmp = document.createElement("input");
  tmp.setAttribute("value", document.getElementById(id).innerHTML);
  document.body.appendChild(tmp);
  tmp.select();
  document.execCommand("copy");
  document.body.removeChild(tmp);
}

function GameInfo(props) {
  if (! props.id)
    return null;

  return (
    <div className="game-info">
      <span className="game-id">{props.id}</span>
      {" "}
      Share this link to invite friends:{" "}
      <span id="game-share-url">{props.url}</span>
      {" "}
      <button
        className="inline"
        onClick={() => copy_to_clipboard('game-share-url')}
      >Copy Link</button>
    </div>
  );
}

function PlayerInfo(props) {
  if (! props.player.id)
    return null;

  return (
    <div className="player-info">
      <span className="players-header">Players: </span>
      {props.player.name}
      {props.other_players.map(other => (
        <span key={other.id}>
          {", "}
          {other.name}
        </span>
      ))}
    </div>
  );
}

function fetch_method_json(method, api = '', data = {}) {
  const response = fetch(api, {
    method: method,
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(data)
  });
  return response;
}

function fetch_post_json(api = '', data = {}) {
  return fetch_method_json('POST', api, data);
}

async function fetch_put_json(api = '', data = {}) {
  return fetch_method_json('PUT', api, data);
}

class CategoryRequest extends React.Component {
  constructor(props) {
    super(props);
    this.category = React.createRef();

    this.handle_change = this.handle_change.bind(this);
    this.handle_submit = this.handle_submit.bind(this);
  }

  handle_change(event) {
    const category_input = this.category.current;
    const category = category_input.value;

    if (/[0-9]/.test(category))
      category_input.setCustomValidity("");
  }

  handle_submit(event) {
    const form = event.currentTarget;
    const category_input = this.category.current;
    const category = category_input.value;

    /* Prevent the default page-changing form-submission behavior. */
    event.preventDefault();

    const match = category.match(/[0-9]+/);
    if (match === null) {
      category_input.setCustomValidity("Category must include a number");
      form.reportValidity();
      return;
    }

    fetch_post_json("prompts", {
      items: parseInt(match[0], 10),
      prompt: category
    });

    form.reset();
  }

  render() {
    return (
      <div className="category-request">
        <h2>Submit a Category</h2>
        <p>
          Suggest a category to play. Don't forget to include the
          number of items for each person to submit.
        </p>

        <form onSubmit={this.handle_submit} >
          <div className="form-field large">
            <input
              type="text"
              id="category"
              placeholder="6 things at the beach"
              required
              autoComplete="off"
              onChange={this.handle_change}
              ref={this.category}
            />
          </div>

          <div className="form-field large">
            <button type="submit">
              Send
            </button>
          </div>

        </form>
      </div>
    );
  }
}

function PromptOptions(props) {

  function handle_click(id) {
    fetch_post_json(`vote/${id}`);
  }

  if (props.prompts.length === 0)
    return null;

  return (
    <div className="prompt-options">
      <h2>Vote on Categories</h2>
      <p>
        Select any categories below that you'd like to play.
        You can choose as many as you'd like.
      </p>
      {props.prompts.map(p => {
        return (
          <button
            className="vote-button"
            key={p.id}
            onClick={() => handle_click(p.id)}
          >
            {p.prompt}
          </button>
        );
      })}
    </div>
  );
}

class Game extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      game_info: {},
      player_info: {},
      other_players: [],
      prompts: []
    };
  }

  set_game_info(info) {
    this.setState({
      game_info: info
    });
  }

  set_player_info(info) {
    this.setState({
      player_info: info
    });
  }

  set_other_player_info(info) {
    const other_players_copy = [...this.state.other_players];
    const idx = other_players_copy.findIndex(o => o.id === info.id);
    if (idx >= 0) {
      other_players_copy[idx] = info;
    } else {
      other_players_copy.push(info);
    }
    this.setState({
      other_players: other_players_copy
    });
  }

  add_or_update_prompt(prompt) {
    const prompts_copy = [...this.state.prompts];
    const idx = prompts_copy.findIndex(p => p.id === prompt.id);
    if (idx >= 0) {
      prompts_copy[idx] = prompt;
    } else {
      prompts_copy.push(prompt);
    }
    this.setState({
      prompts: prompts_copy
    });
  }

  render() {
    const state = this.state;

    return [
      <GameInfo
        key="game-info"
        id={state.game_info.id}
        url={state.game_info.url}
      />,
      <PlayerInfo
        key="player-info"
        game={this}
        player={state.player_info}
        other_players={state.other_players}
      />,
      <p key="spacer"></p>,
      <CategoryRequest
        key="category-request"
      />,
      <PromptOptions
        key="prompts"
        prompts={state.prompts}
      />
    ];
  }
}

ReactDOM.render(<Game
                  ref={(me) => window.game = me}
                />, document.getElementById("empathy"));
